import {v4}from'uuid';import {ClientTypes,getGlobalVariable}from'@common-stack/core';import {merge}from'lodash-es';import {ScopedContainer}from'@common-stack/client-core/lib/connector/ScopedContainer.js';import {createReduxStore}from'../../config/redux-config.js';import {logger}from'@cdm-logger/server';import {UtilityClass}from'../../utils/index.js';import features from'../../modules.js';import {createApolloClient}from'../../config/base-apollo-client.js';import {config}from'../../config/env-config.server.js';const TYPES = {
    HttpRequest: Symbol.for('HttpRequest'),
};
const utility = new UtilityClass(features);
ScopedContainer.registerGlobalDependencies((container) => {
    container.bind(ClientTypes.Logger).toConstantValue(logger);
    container.bind(ClientTypes.UtilityClass).toConstantValue(utility);
});
const containerMiddleware = async (req, res, next) => {
    const requestId = v4(); // Generate a unique identifier for the request
    if (config.LOG_LEVEL === 'debug' || config.LOG_LEVEL === 'trace') {
        console.time(`Request ${requestId} Duration`); // Start timing the request
    }
    logger.debug(`Express req started - ID: ${requestId} , URL: ${req.url}`); // Log when request starts with unique ID
    try {
        const container = features.createContainers({}, requestId);
        const clientState = features.getStateParams({
            scopeId: requestId,
            requestResponsePair: {
                req,
                res,
            },
        });
        const { apolloClient } = createApolloClient({
            httpGraphqlURL: config.GRAPHQL_URL,
            httpLocalGraphqlURL: config.LOCAL_GRAPHQL_URL,
            isDev: process.env.NODE_ENV === 'development',
            isDebug: getGlobalVariable('__DEBUGGING__'),
            isSSR: getGlobalVariable('__SSR__'),
            scope: typeof window !== 'undefined' ? 'browser' : 'server',
            clientState,
            getDataIdFromObject: (result) => features.getDataIdFromObject(result),
            initialState: typeof window !== 'undefined' ? window?.__APOLLO_STATE__ : undefined,
            logger,
        });
        container.bind(TYPES.HttpRequest).toConstantValue(req);
        let count = 1;
        container
            .bind(ClientTypes.ApolloClientFactory)
            .toDynamicValue((context) => () => {
            count = count + 1;
            logger.debug(`Apollo Client Factory called - Count: ${count}`);
            return apolloClient;
        })
            .inRequestScope();
        const services = merge({}, ...features.createServiceFunc.map((serviceFunc) => serviceFunc(container)));
        const resolvers = features.getApolloResolvers(() => services);
        apolloClient.setResolvers(resolvers);
        const { store } = createReduxStore(apolloClient, services, container);
        req.container = container;
        req.apolloClient = container.get(ClientTypes.ApolloClientFactory)();
        req.logger = logger;
        req.store = store;
        req.services = services;
        if (typeof res.on === 'function') {
            res.on('finish', () => {
                logger.debug(`Express req Finished! - ID: ${requestId}`); // Log when request finishes with unique ID
                if (config.LOG_LEVEL === 'debug' || config.LOG_LEVEL === 'trace') {
                    console.timeEnd(`Request ${requestId} Duration`); // End timing and log the duration
                }
                try {
                    ScopedContainer.remove(requestId);
                    req.apollClient = null;
                    req.services = null;
                    logger.debug(`Successfully cleaned up - ID: ${requestId}`);
                }
                catch (error) {
                    logger.error(error, 'Error during container cleanup - ID: [%s]', requestId);
                }
            });
        }
        return await next();
    }
    catch (error) {
        logger.error(`Error processing request - ID: ${requestId}`, error);
        return await next(error);
    }
};export{TYPES,containerMiddleware};